#ifndef _Constants_H
#define _Constants_H

// for debugging we can include code, but makes computation slower
//#define DEBUG 1
//
// please do not change EXPERIMENTAL unless you know what you do
//   it enables/disables fallback or test versions for the developpers
//   combine different experimental options using bitwise OR
//#define EXPERIMENTAL 1  // fast recursive config generation (bravais only?)
//                         buggy! 2010-01
//#define EXPERIMENTAL 2  // recompute terms and transfer via mkfifo + pthread
//#define EXPERIMENTAL 4 // estimate maxGAP instead of computation (fast)
//#define EXPERIMENTAL 8  // speedup 2 on big systems but asymmetric expects JS1002
//                      // xxzLC-12 75s to 43s v1002
#define EXPERIMENTAL (8+4)
#define TermCompression (2+1) // saves memory, may give less accuracy at 1e-7
// JS1002 ToDo: compress via index list to double values !? TermCompression=2

// dimension of model 0=finite 1=chain ... 3=3D (slow)
// lower values reduce computation effort in the inner loops
#define Dimension 3
//#define SymInt1D 1  // use faster integer symmetries for linear chain
//#define SymInt2D 1  // use faster integer symmetries for square lattice
//#define SymIntSinghHuse6 1 // test code for singh_huse_triangle_N_6.ccm

// int Array[const int x=5] works only for C++
#define MAX_UNIT_CELL 6  // CaVO=4, kagome=3, ..._N_6=6 etc.
// #define MAX_UNIT_CELL 1  // speedup for Bravais lattices
#define MAX_NUM_SYM   8  // max=8 for xxzSquare.ccm
//#define MAX_NUM_SYM  48  // max=48 for xxzCubic.ccm

// size of the descriptor index in units of integer size, smaller is faster
//   this should be about 2spinlen*MaxArraySize/32
//   set to minimum for max speed
//   ToDo: use bits per site + DESCR_SIZE= max_nuc*uc_sites*bits_per_spin/32
//         rename BSize to Sites_per_INT ???
// JS0911: MAX_SPIN_LIST renamed to DESCR_SIZE (better name)
//#define DESCR_SIZE 1    // 1D(s=1/2,m=31), 1D(s=1,m=15), sq(s=1/2,m=7)
//#define DESCR_SIZE 2    // 1D(s=1/2,m=63), 1D(s=1,m=31), sq(s=1/2,m=9)?
#define DESCR_SIZE 3    // cavo_striped-6-6,UC4 (make check)
//#define DESCR_SIZE 6  // square(s=1/2,m=12)
//#define DESCR_SIZE 8  // stagg_spiral, triangle_f1 (s=1/2,boundary=12)
//#define DESCR_SIZE 14   // triangle_f1 (s=1/2,boundary=16)
//#define DESCR_SIZE 58   // sq(s=1,m=28?) kago(s=1,m=16?)
//#define DESCR_SIZE 166  // cubic(s=4,m=8) 

// store config as 2bit (2s=1) per site (16sites/int*MAX_SPIN_LIST)
// or (ToDo) max-n flips * (x,y,z,uc) * char..short

// MAX_NUM_CONFIGURATIONS.max=2^(32-8)=16e6 (see storage.C term compression!)
// MAX_NUM_CONFIGURATIONS < 2^15 (for best term compression of 50%)
// MAX_NUM_CONFIGURATIONS 1500000 (square LSUB 12 excitations)
#define MAX_NUM_CONFIGURATIONS 1500000
// v0903 temporary needs above size to alloc, than realloc to needed size

#define MAX_NUM_NEIGHBOURS 10

//  may consume unneccessary lot of memory
//  ToDo: remove and replace by dynamic allocations
#define MAX_SUBM 30  // max distance of flips on the lattice
#define MAX_SUBN 28  // max num spin flips (v1002: maximum is 28, binary_max)

#define MAX_NUM_ITERATIONS 5000  // Mar08: reduced from 10000 to 4000
#define TOLERANCE 1e-10  // old= 1e-12 to strong? (double_prec=1e-13)
//#define TOLERANCE 1e-12  // old= 1e-12 to strong? (double_prec=1e-13)

/* momentum of convergence for direct iteration
 * JS-Jan2010: removed, see options */
// #define momentum  0.75 // 0.625 0.75  old: 0.7 (look at j1j2_square_Neel)

// if no FLAGS defined, define for serial computation now
#ifndef PARALLEL_FLAG      // old longer macro name (obsolete)
#ifdef CFG_MPI             // new short macro name
#define PARALLEL_FLAG CFG_MPI
#else
#define PARALLEL_FLAG 0
#endif
#endif

#ifndef CFG_MPI
#define CFG_MPI PARALLEL_FLAG
#endif

#if CFG_MPI != PARALLEL_FLAG
#error Macros _CFG_MPI and _PARALLEL_FLAG differ
#endif

#ifndef MPICH_IGNORE_CXX_SEEK  // go around mpich2 compile bug
#define MPICH_IGNORE_CXX_SEEK
#endif

// JS2011-07 set 1: exit the program in case of non-convergence
//   instead of going further (set 0), wished by Peggy
#ifndef EXIT_ON_ERROR
#define EXIT_ON_ERROR 0   // 0 is default, 1 for exit on error
#endif

#endif
